/** framen
    
    [

    based on Rob Bothof's modification of the speaker_pcm arduino example by Michael Smith <michael@hurts.ca>
    http://playground.arduino.cc/Code/PCMAudio

    Finetuned by Leoš Hort
    https://www.facebook.com/HRTLmuzik
    
    Stole the idea from:
    https://github.com/thatpixguy/hrtl-amen
    (+ nice sox oneliner)
    sox input.wav -t u8 -c 1 -r 8000 - | xxd -i
    haven't tried but probably works too http://highlowtech.org/?p=1963
 
    ]

    modified by Robert Beenen <robert@foresense.org>
       
  * added loop overflow to start of sample. this means 'loop start' never affects 'loop length'.
  * added a gate input to retrigger the sample. pitch is performance only. might work that back in.
  * sanitized the parameter rangesm, tightened up the sound.
  * restructured the code a bit, moved sample data to sample.h so you can swap in some new sound easily.

    KNOB 1: loop start
    KNOB 2: loop length
    KNOB 3: pitch
    INPUT 3: gate trigger

  ***/

#include <stdint.h>
#include <avr/interrupt.h>
#include <avr/io.h>
#include <avr/pgmspace.h>

// sound_data & sound_length
#include "sample.h"

#define LED_PIN     13
#define SPEAKER_PIN 11  // This is fixed on the Grains to 11 

#define KNOB_1  2   // coupled with IN_1, sample start (percentage of whole sample)
#define KNOB_2  1   // coupled with IN_2, loop size    (percentage of whole sample)
#define KNOB_3  0   // pitch
#define INPUT_3 3   // gate trigger

volatile uint16_t sample;
volatile uint16_t loop_start;
volatile uint16_t loop_length;
volatile uint16_t bounds;
volatile uint16_t overflow;

volatile uint16_t gate;
volatile uint16_t gate_prev;

byte lastSample;

void startPlayback()
{
    pinMode(SPEAKER_PIN, OUTPUT);

    // Set up Timer 2 to do pulse width modulation on the speaker pin.
    // Use internal clock (datasheet p.160)
    ASSR &= ~(_BV(EXCLK) | _BV(AS2));

    // Set fast PWM mode  (p.157)
    TCCR2A |= _BV(WGM21) | _BV(WGM20);
    TCCR2B &= ~_BV(WGM22);

    // Do non-inverting PWM on pin OC2A (p.155)
    // On the Arduino this is pin 11.
    TCCR2A = (TCCR2A | _BV(COM2A1)) & ~_BV(COM2A0);
    TCCR2A &= ~(_BV(COM2B1) | _BV(COM2B0));
    // No prescaler (p.158)
    TCCR2B = (TCCR2B & ~(_BV(CS12) | _BV(CS11))) | _BV(CS10);

    // Set initial pulse width to the first sample.
    OCR2A = pgm_read_byte(&sound_data[0]);

    // Set up Timer 1 to send a sample every interrupt.
    cli();

    // Set CTC mode (Clear Timer on Compare Match) (p.133)
    // Have to set OCR1A *after*, otherwise it gets reset to 0!
    TCCR1B = (TCCR1B & ~_BV(WGM13)) | _BV(WGM12);
    TCCR1A = TCCR1A & ~(_BV(WGM11) | _BV(WGM10));

    // No prescaler (p.134)
    TCCR1B = (TCCR1B & ~(_BV(CS12) | _BV(CS11))) | _BV(CS10);

    // Set the compare register (OCR1A).
    // OCR1A is a 16-bit register, so we have to do this with
    // interrupts disabled to be safe.
    OCR1A = F_CPU / SAMPLE_RATE; // 16e6 / 8000 = 2000

    // Enable interrupt when TCNT1 == OCR1A (p.136)
    TIMSK1 |= _BV(OCIE1A);

    lastSample = pgm_read_byte(&sound_data[sound_length - 1]);
    sample = 0;
    sei();
}

void stopPlayback()
{
    TIMSK1 &= ~_BV(OCIE1A); // Disable playback per-sample interrupt.
    TCCR1B &= ~_BV(CS10);   // Disable the per-sample timer completely.
    TCCR2B &= ~_BV(CS10);   // Disable the PWM timer.
    digitalWrite(SPEAKER_PIN, LOW);
}

void setup()
{
    pinMode(LED_PIN, OUTPUT);
    digitalWrite(LED_PIN, HIGH);

    startPlayback();

    loop_start  = 0; 
    loop_length = sound_length;
}

// This is called at 8000 Hz to load the next sample.
ISR(TIMER1_COMPA_vect)
{
    if(sample >= bounds)
    {
        sample = loop_start;
    }
    else if((sample < loop_start) &&
            (sample >= overflow))
    {
        sample = loop_start;
    }
    else
    {
        OCR2A = pgm_read_byte(&sound_data[sample % sound_length]);
    }
    ++sample;
}

void loop()
{
    loop_start = analogRead(KNOB_1) / 1024.0 * sound_length;
    loop_length = (analogRead(KNOB_2) + 1) / 1024.0 * sound_length;
//  bounds can be up to 2x sound length. the more you know.
    bounds = loop_start + loop_length;
//  this will set the overflow length 
//  to take the looplength overflow into account when
//  checking the loop boundaries
    if(bounds > sound_length)
    {
        overflow = bounds - sound_length;
    }
//  rising signal = trigger sample reset.. slightly buffered
    gate = analogRead(INPUT_3);
    if(gate > gate_prev)
    {
        sample = loop_start;
    }
    gate_prev = gate;
    OCR1A = (512.0 / (analogRead(KNOB_3) + 1)) * (F_CPU / SAMPLE_RATE);
}
